/**
 * ProtsAndPepsDataHandler.java
 *
 * Created on Aug 1, 2007, 10:07:42 AM
 * @author breuk102
 *  This data handler will take the generic ProtsAndPeps data and stores the column information where to find
 *  the peptide sequence, Signal A, Signal B etc.
 */
package nl.uu.chem.bioinformatics.statquant;

import java.io.File;
import java.io.FileNotFoundException;
import nl.uu.chem.bioinformatics.statquant.model.QuantifiedAminoAcidSequence;
import nl.uu.chem.bioinformatics.statquant.parsers.GenericParser;
import nl.uu.chem.bioinformatics.statquant.parsers.MSQuantParser2;
import nl.uu.chem.bioinformatics.statquant.parsers.MSQuantParser;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Iterator;
import org.apache.commons.math.MathException;
import org.apache.commons.math.stat.inference.TestUtils;
import jxl.*;
import jxl.format.UnderlineStyle;
import jxl.write.*;
import jxl.write.Number;

/**
 * get an instance of the ProtsAndPepsDatHandler
 */
public class ProtsAndPepsDataHandler {

    private HashMap<String, String[]> ProteinInfo = new HashMap<String, String[]>();
    private QuantifiedAminoAcidSequence quantifiedSequence;
    private HashMap<String, ArrayList<QuantifiedAminoAcidSequence>> ProtsAndPeps = new HashMap<String, ArrayList<QuantifiedAminoAcidSequence>>();
    private ArrayList<Class> parserArray = new ArrayList<Class>();
    // Next are the fields that are absolutely required for proper MSQ file parsing. Default values are inserted
    private int peptideColumn = 8; //starting from 0!
    private int peptideSignalA = 91;
    private int peptideSignalB = 96;
    private int proteinNameColumn = 2;
    private int modificationColumn = 36;
    private int protDescriptionColumn = 8;
    private int PTMpositionColNumber = 0;
    private int ScanNumberColNumber = 0;
    private int QueryColNumber = 0;
    /** QueryNnumber */
    private int pepScoreColNumber = 0;
    /** Score for the peptide */
    private boolean isPConverted = false;
    private float conversionFactor = 0f;
    private boolean useUniquePeptidesOnly = false;
    private boolean usePeptideReport = false;
    private boolean filteredForUniqueness = false;
    private boolean filteredForUniqueness_show = false;
    private String fileName = "";
    private ArrayList<String[]> headers = new ArrayList<String[]>();
    // Parser options
    private String useParserName = "MSQuantParser";
    private float useParserVersion = 0.1f;

    // REGISTER PARSERS HERE
    private void registerParsers() {
        parserArray.add(MSQuantParser.class);
        parserArray.add(MSQuantParser2.class);
        parserArray.add(GenericParser.class);
    }

    /**
     * This function starts another function ins the MSQparser that tries to determine
     * the columns without opening the complete file.
     * it reads up to the header and adds this header in the MSQ class. It also stores the
     * column numbers. Next we pick up the header and column numbers and use them in the display
     * to the user. let him/her decide if they are ok.
     */
    public void getMSQColumns() {
        //this column finder works for both the normal als version 2parser!
        MSQuantParser2 MSQP = new MSQuantParser2();
        if (fileName.equals("")) {
            System.out.println("Could not parse, no filename was given");
        } else {
            try {
                MSQP.setFileName(fileName);
                MSQP.determineColumnsFromFirstRead();
                headers = MSQP.getHeaders();
                setProteinNameColumn(MSQP.getProteinNameColumn());
                setPeptideColumn(MSQP.getPeptideColumn());
                setPeptideSignalA(MSQP.getPeptideSignalA());
                setPeptideSignalB(MSQP.getPeptideSignalB());
                setModificationColumn(MSQP.getModificationColumn());
            } catch (IOException ex) {
                //Logger.getLogger("global").log(Level.SEVERE, null, ex);
            }
        }
    }

    public void runParser() {
        startParser();
    }

    public String getProteinDescription(String protID) {

        if (isUsePeptideReport()) {


            return ProtsAndPeps.get(protID).get(0).getPeptideinfo()[protDescriptionColumn] + " \t " + ProtsAndPeps.get(protID).get(0).getPeptideinfo()[headerIndex("Description")];

        } else {
            return ProtsAndPeps.get(protID).get(0).getPeptideinfo()[protDescriptionColumn];
        }
    }

    public void runParser(String ParserName) {
        setUseParserName(ParserName);
        startParser();
    }

    // This is the actual Parser Selection Area:
    // Not very pro, but it makes pluggin in new parser possible without tooo much hassle
    // TODO: MAke this more professional
    private void startParser() {
        if (useParserName.equals("MSQuantParser")) {
            startMSQParser();
        } else if (useParserName.equals("generic")) {
            //System.out.println("A generic parser needs to be implemented yet");
            startGenericParser();
        } else if (useParserName.equals("MSQuantParser2")) {
            startMSQParser2();
        } else {
            System.out.println("no parser was given");
        }
    }

    @SuppressWarnings(value = "unchecked")
    private void startMSQParser() {
        // Pass All variables
        MSQuantParser MSQP = new MSQuantParser();
        MSQP.setProtsAndPeps(ProtsAndPeps); // Re-use the proteins and peptides already found. This way we ADD the data from several files. If you do not need  well do: new blabladatahandle();
        MSQP.setProteinInfo(ProteinInfo); // idem
        MSQP.setPeptideColumn(peptideColumn);
        MSQP.setPeptideSignalA(peptideSignalA);
        MSQP.setPeptideSignalB(peptideSignalB);
        MSQP.setModificationColumn(modificationColumn);
        MSQP.setProteinNameColumn(proteinNameColumn);
        MSQP.setUseUniquePeptidesOnly(useUniquePeptidesOnly);
        MSQP.setPTMpositionColNumber(PTMpositionColNumber);

        if (usePeptideReport) {
            MSQP.setUsePeptideReport(usePeptideReport);
        }


        if (fileName.equals("")) {
            System.out.println("Could not parse, no filename was given");
        } else {
            MSQP.setFileName(fileName);
            MSQP.run();
            ProtsAndPeps = MSQP.getProtsAndPeps();
            ProteinInfo = MSQP.getProteinInfo();
        }
        headers = MSQP.getHeaders();
    }

    @SuppressWarnings(value = "unchecked")
    private void startMSQParser2() {
        // Pass All variables
        MSQuantParser2 MSQP = new MSQuantParser2();
        MSQP.setProtsAndPeps(ProtsAndPeps); // Re-use the proteins and peptides already found. This way we ADD the data from several files. If you do not need  well do: new blabladatahandle();
        MSQP.setProteinInfo(ProteinInfo); // idem
        MSQP.setPeptideColumn(peptideColumn);
        MSQP.setPeptideSignalA(peptideSignalA);
        MSQP.setPeptideSignalB(peptideSignalB);
        MSQP.setModificationColumn(modificationColumn);
        MSQP.setProteinNameColumn(proteinNameColumn);
        MSQP.setUseUniquePeptidesOnly(useUniquePeptidesOnly);
        MSQP.setPTMpositionColNumber(PTMpositionColNumber);
        if (usePeptideReport) {
            MSQP.setUsePeptideReport(usePeptideReport);
        }

        if (fileName.equals("")) {
            System.out.println("Could not parse, no filename was given");
        } else {
            MSQP.setFileName(fileName);
            MSQP.run();
            ProtsAndPeps = MSQP.getProtsAndPeps();
            ProteinInfo = MSQP.getProteinInfo();
        }
        headers = MSQP.getHeaders();
    }

    /**
     * A function to save the contents of the ProtsAndPepsData with the statistical end results as shown by the main table and the peptable
     * @filename= File name of the file to save to
     * @return= a boolean value, true is successfull
     */
    public void saveTotalReport(String filename) throws FileNotFoundException {

        FileOutputStream out; // declare a file output object
        PrintStream p; // declare a print stream object

        // Create a new file output stream
        // connected to "myfile.txt"
        out = new FileOutputStream(filename + "_report.txt");
        // Connect print stream to the output stream
        p = new PrintStream(out);
        Iterator<String> it = ProtsAndPeps.keySet().iterator();
        int counter = 1;
        while (it.hasNext()) {
            String Prot = it.next();
            float ratio = getProteinRatio(Prot);
            float STDEV = getProteinSTDEV(Prot);
            float logSTDEV = getProteinLogSTDEV(Prot);
            float logratio = getProteinLogRatio(Prot);
            float pvalue = getProteinLogPvalue(Prot, 2, 0.5f, 0d);
            int nrpeps = getProteinPeptideNumber(Prot);
            String descriptions = getProteinDescription(Prot);
            counter++;
            p.println("nr \t ID \t #peptides \t ratio \t STDEV \t logRatio \t logSTDEV \t pValue \tDescription");
            p.println(counter + "\t" + Prot + "\t" + nrpeps + "\t" + ratio + "\t" + STDEV + "\t" + logratio + "\t" + logSTDEV + "\t" + pvalue + "\t" + descriptions);
            p.println("\tID\tsequence\tmodification\tpeptide ratio\tchecked\tPTMinfo\tnr prolines\tpro Ratio\tScan #");
            // next is to get all the QA values we want ;)
            for (QuantifiedAminoAcidSequence QA : ProtsAndPeps.get(Prot)) {
                String Seq = QA.getSequence();
                float pepRatio = QA.getRatio();
                boolean checked = QA.isChecked();

                String checkedPTM = "";
                if (getPTMpositionColNumber() > 0) {
                    checkedPTM = QA.getSeqWithMods();
                }

                String ScanNumber = "";
                if (getScanNumberColNumber() > 0) {
                    ScanNumber = QA.getPeptideinfo()[getScanNumberColNumber()];
                }

                int nrpro = 0;
                float oldRatio = 0;
                if (isIsPConverted()) {
                    // show additional info in peptide table
                    nrpro = getAminoAcidOccurence("P", QA.getSequence());
                    oldRatio = prolineConversion(QA.getSequence(), getConversionFactor());
                }
                String mods = QA.getModification();

                p.println("\t" + Prot + "\t" + Seq + "\t" + mods + "\t" + pepRatio + "\t" + checked + "\t" + checkedPTM + "\t" + nrpro + "\t" + oldRatio + "\t" + ScanNumber);
            }
            p.println();
        }
        p.close();
    }

    /**
     *
     * @param filename
     */
    public void saveReportAsExcel(String filename, String sheetName, boolean includePeptides) throws IOException, WriteException {

        if (!filename.endsWith(".xls")) {
            filename = filename + ".xls";
        }

        WritableWorkbook workbook = Workbook.createWorkbook(new File(filename));
        WritableSheet sheet = workbook.createSheet(sheetName, 0);

        WritableFont headerFont = new WritableFont(WritableFont.ARIAL, 14, WritableFont.NO_BOLD, false, UnderlineStyle.NO_UNDERLINE, jxl.format.Colour.BLACK);
        WritableCellFormat headerFormat = new WritableCellFormat(headerFont);

        WritableFont remarkFont = new WritableFont(WritableFont.ARIAL, 10, WritableFont.NO_BOLD, false, UnderlineStyle.NO_UNDERLINE, jxl.format.Colour.GREY_50_PERCENT);
        WritableCellFormat remarkFormat = new WritableCellFormat(remarkFont);

        WritableFont font = new WritableFont(WritableFont.ARIAL, 11);
        WritableCellFormat defaultFormat = new WritableCellFormat(font);

        int counter = 0;
        int row = 0;

        String[] protHeadernames = new String[]{"no", "Id", "#peptides", "ratio", "stdev", "log ratio", "log ratio stdev", "p-value", "description", "Signal A sum", "Signal B sum"};
        String[] pepHeadernames = new String[]{"ID", "sequence", "modification", "peptide ratio", "checked", "PTMinfo", "nr prolines", "pro Ratio", "Scan number", "Signal A", "Signal B"};

        // HEADERS

        for (int col = 0; col < protHeadernames.length; col++) {
            sheet.addCell(new Label(col, row, protHeadernames[col], headerFormat));
        }

        row++;

        if (includePeptides) {
            for (int col = 0; col < pepHeadernames.length; col++) {
                sheet.addCell(new Label(col + 1, row, pepHeadernames[col], headerFormat)); //offset by 1
            }
            row++;
        }

        for (int col = 0; col < protHeadernames.length; col++) {
            sheet.addCell(new Label(col, row, protHeadernames[col], headerFormat));
        }

        // DATA

        for (String protName : ProtsAndPeps.keySet()) {

            counter++;
            int protcol = 0;

            int nrpeps = getProteinPeptideNumber(protName);
            float ratio = getProteinRatio(protName);
            float STDEV = getProteinSTDEV(protName);
            float logratio = getProteinLogRatio(protName);
            float logSTDEV = getProteinLogSTDEV(protName);
            float pvalue = getProteinLogPvalue(protName, 2, 0.5f, 0d);
            String descriptions = getProteinDescription(protName);

            sheet.addCell(new Number(protcol++, row, counter, headerFormat));
            sheet.addCell(new Label(protcol++, row, protName, defaultFormat));
            sheet.addCell(new Number(protcol++, row, nrpeps, defaultFormat));
            sheet.addCell(new Number(protcol++, row, ratio, defaultFormat));
            sheet.addCell(new Number(protcol++, row, STDEV, defaultFormat));
            sheet.addCell(new Number(protcol++, row, logratio, defaultFormat));
            sheet.addCell(new Number(protcol++, row, logSTDEV, defaultFormat));
            sheet.addCell(new Number(protcol++, row, pvalue, defaultFormat));
            sheet.addCell(new Label(protcol++, row, descriptions, remarkFormat));

            int proteinrow = row++;
            double sigAsum = 0;
            double sigBsum = 0;

            // next is to get all the QA values we want ;)

            for (QuantifiedAminoAcidSequence quantifiedPeptide : ProtsAndPeps.get(protName)) {
                int col = 1;

                boolean checked = quantifiedPeptide.isChecked();
                WritableCellFormat localCellFormat = checked ? defaultFormat : remarkFormat;
                String sequence = quantifiedPeptide.getSequence();
                String mods = quantifiedPeptide.getModification();
                float pepRatio = quantifiedPeptide.getRatio();

                String checkedPTM = "";
                if (getPTMpositionColNumber() > 0) {
                    checkedPTM = quantifiedPeptide.getSeqWithMods();
                }

                int nrpro = 0;
                float oldRatio = 0;
                if (isIsPConverted()) {
                    // show additional info in peptide table
                    nrpro = getAminoAcidOccurence("P", quantifiedPeptide.getSequence());
                    oldRatio = prolineConversion(quantifiedPeptide.getSequence(), getConversionFactor());
                }

                String scanNumber = "";
                if (getScanNumberColNumber() > 0) {
                    scanNumber = quantifiedPeptide.getPeptideinfo()[getScanNumberColNumber()];
                }

                double signalA = 0;
                if (getPeptideSignalA() > 0) {
                    signalA = Double.parseDouble(quantifiedPeptide.getPeptideinfo()[getPeptideSignalA()]);
                    sigAsum += signalA;
                }

                double signalB = 0;
                if (getPeptideSignalB() > 0) {
                    signalB = Double.parseDouble(quantifiedPeptide.getPeptideinfo()[getPeptideSignalB()]);
                    sigBsum += signalB;
                }
                if (includePeptides) {
                    sheet.addCell(new Label(col++, row, protName, defaultFormat));
                    sheet.addCell(new Label(col++, row, sequence, localCellFormat));
                    sheet.addCell(new Label(col++, row, mods, localCellFormat));
                    sheet.addCell(new Number(col++, row, pepRatio, localCellFormat));
                    sheet.addCell(new Label(col++, row, checked ? "" : "deselected", localCellFormat));
                    sheet.addCell(new Label(col++, row, checkedPTM, localCellFormat));
                    sheet.addCell(new Number(col++, row, nrpro, localCellFormat));
                    sheet.addCell(new Number(col++, row, oldRatio, localCellFormat));
                    sheet.addCell(new Label(col++, row, scanNumber, localCellFormat));
                    sheet.addCell(new Number(col++, row, signalA, localCellFormat));
                    sheet.addCell(new Number(col++, row, signalB, localCellFormat));

                    row++;
                }
            }
            sheet.addCell(new Number(protcol++, proteinrow, sigAsum, defaultFormat));
            sheet.addCell(new Number(protcol++, proteinrow, sigBsum, defaultFormat));
        }


        workbook.write();

        workbook.close();
    }

    private void startGenericParser() {
        // Pass All variables
        GenericParser GP = new GenericParser();
        GP.setProtsAndPeps(ProtsAndPeps); // Re-use the proteins and peptides already found. This way we ADD the data from several files. If you do not need  well do: new blabladatahandle();
        GP.setProteinInfo(ProteinInfo); // idem
        GP.setPeptideColumn(peptideColumn);
        GP.setPeptideSignalA(peptideSignalA);
        GP.setPeptideSignalB(peptideSignalB);
        GP.setModificationColumn(modificationColumn);
        GP.setProteinNameColumn(proteinNameColumn);
        GP.setUseUniquePeptidesOnly(useUniquePeptidesOnly);
        GP.setPTMpositionColNumber(0);
        if (usePeptideReport) {
            GP.setUsePeptideReport(usePeptideReport);
        }

        if (fileName.equals("")) {
            System.out.println("Could not parse, no filename was given");
        } else {
            GP.setFileName(fileName);
            GP.run();
            ProtsAndPeps =
                    GP.getProtsAndPeps();
            ProteinInfo =
                    GP.getProteinInfo();
        }

        headers = GP.getHeaders();
    }

    public void getGenericColumns() {
        GenericParser GP = new GenericParser();
        if (fileName.equals("")) {
            System.out.println("Could not parse, no filename was given");
        } else {
            try {
                GP.setFileName(fileName);
                GP.determineColumnsFromFirstRead();
                headers =
                        GP.getHeaders();
                //System.out.println(headers.get(0)[0]);
                setProteinNameColumn(GP.getProteinNameColumn());
                setPeptideColumn(GP.getPeptideColumn());
                setPeptideSignalA(GP.getPeptideSignalA());
                setPeptideSignalB(GP.getPeptideSignalB());
                setModificationColumn(GP.getModificationColumn());
            } catch (IOException ex) {
                //Logger.getLogger("global").log(Level.SEVERE, null, ex);
            }
        }
    }
    // DO THINGS WITH DATA AREA!

    /** The data is of the type: HashMap<String, ArrayList<QuantifiedAminoAcidSequence>> ProtsAndPeps
     *  A Hash(String) with protein ID and a ArrayList with QuantifiedAminoAcidSequence data.
     *  In addition we have the ProteinInformation (the protein and it's annotation)
     */
    public String getPeptideModification(
            String proteinName, int index) {
        return ProtsAndPeps.get(proteinName).get(index).getModification();
    }

    public float prolineConversion(String sequence, float percentage) {
        int nrP = getAminoAcidOccurence("P", sequence);
        float probability = 0f;
        for (int t = 1; t <=
                nrP; t++) {
            probability += binomdist(getAminoAcidOccurence("P", sequence), t, percentage);
        }
// make 100% scake
//probability = probability*100;

        float correctionFactor = 1 + probability;
        return correctionFactor;
    }

    public void correctProlineConversion(String proteinName, float percentage) {
        for (QuantifiedAminoAcidSequence QA : ProtsAndPeps.get(proteinName)) {

            // if (QA.isChecked()) {

            // get the sequence and find number of prolines
            int nrP = getAminoAcidOccurence("P", QA.getSequence());
            // the correction needs to be applied on the area of the heavy label.
            // this should be SIGNAL 2.. get it
            float oldArea = Float.parseFloat(QA.getPeptideinfo()[peptideSignalB]);
            float probability = 0f;
            for (int t = 1; t <=
                    nrP; t++) {
                probability += binomdist(getAminoAcidOccurence("P", QA.getSequence()), t, percentage);
            }
// make 100% scake
//probability = probability*100;

            float correctionFactor = 1 + probability;

            float newArea = oldArea * correctionFactor;
            float newRatio = newArea / Float.parseFloat(QA.getPeptideinfo()[peptideSignalA]);
            // System.out.println("Seq "+QA.getSequence()+" nrP: "+nrP + " Prob "+probability +" Ratio "+newRatio+" corr:"+correctionFactor+" oldA "+oldArea+" NewA "+newArea);
            // inject this new Ratio Back into the QA
            QA.getPeptideinfo()[getPeptideSignalB()] = String.valueOf(newArea);
            QA.setRatio(newRatio);
            //  }
        }

    }

    // N is nr of for example Prolines
    // n is amount of possible configurations
    // occurence = the chance of finding a certain aminoacid eg, the % of P' that have been converted eg 18% = 0.18
    private float binomdist(int N, int n, float occurence) {
        float Theta = occurence / (1 - occurence);
        double chance = calculateFaculty(N) / (calculateFaculty(n) * calculateFaculty(N - n)) * java.lang.Math.pow((double) Theta, (double) n) * java.lang.Math.pow(1 + (double) Theta, 0 - (double) N);
        return (float) chance;
    }

    private float calculateFaculty(int x) {
        float result = 1f;
        for (int t = 1; t <=
                x; t++) {
            result = result * t;
        }

        return result;
    }

// this function return the number/occurence of an aminoacid in a peptide string
    public int getAminoAcidOccurence(String AA, String sequence) {
        int count = 0;
        for (int t = 0; t <
                sequence.length(); t++) {
            String X = String.valueOf(sequence.charAt(t));
            if (AA.equals(X)) {
                count++;
            }

        }
        return count;
    }

    public float getProteinRatio(String proteinName) {
        float protRatio = 0f;
        int counter = 0;
        float totalRatio = 0f;
        for (QuantifiedAminoAcidSequence QA : ProtsAndPeps.get(proteinName)) {
            if (QA.isChecked()) {
                totalRatio += QA.getRatio();
                counter++;

            }






        }
        protRatio = totalRatio / counter;
        return protRatio;
    }

    public float getProteinLogRatio(String proteinName) {
        float protRatio = 0f;
        int counter = 0;
        float totalRatio = 0f;
        for (QuantifiedAminoAcidSequence QA : ProtsAndPeps.get(proteinName)) {
            if (QA.isChecked()) {
                totalRatio += QA.getLogRatio();
                counter++;

            }






        }
        protRatio = totalRatio / counter;
        return protRatio;
    }

    public float getProteinLogSTDEV(String proteinName) {
        float x = 0f;
        float x2 = 0f;
        float SS = 0f;
        int n = 0;
        float stdev = 0f;
        float totalRatio = 0f;
        for (QuantifiedAminoAcidSequence QA : ProtsAndPeps.get(proteinName)) {
            if (QA.isChecked()) {
                x += QA.getLogRatio();
                x2 +=
                        QA.getLogRatio() * QA.getLogRatio();
                n++;

            }






        }
        float xave = x / n;
        float x2ave = x2 / n;
        SS =
                x2 - ((x * x) / n);
        float variance = SS / (n - 1);
        stdev =
                (float) java.lang.Math.sqrt(variance);
        return stdev;
    }

    public float getProteinSTDEV(String proteinName) {
        float x = 0f;
        float x2 = 0f;
        float SS = 0f;
        int n = 0;
        float stdev = 0f;
        float totalRatio = 0f;
        for (QuantifiedAminoAcidSequence QA : ProtsAndPeps.get(proteinName)) {
            if (QA.isChecked()) {
                x += QA.getRatio();
                x2 +=
                        QA.getRatio() * QA.getRatio();
                n++;

            }






        }
        float xave = x / n;
        float x2ave = x2 / n;
        SS =
                x2 - ((x * x) / n);
        float variance = SS / (n - 1);
        stdev =
                (float) java.lang.Math.sqrt(variance);
        return stdev;
    }

    public int getProteinPeptideNumber(String proteinName) {
        int nrpeptides = 0;
        for (QuantifiedAminoAcidSequence QA : ProtsAndPeps.get(proteinName)) {
            if (QA.isChecked()) {
                nrpeptides++;
            }

        }
        return nrpeptides;
    }

    public ArrayList<QuantifiedAminoAcidSequence> getQuantifiedPeptides(String proteinName) {
        //TODO: BUILD CHECK IF ELEMENT EXISTS
        return ProtsAndPeps.get(proteinName);
    }

    public float getProteinTstatistics(float STDEV, int n, float averageRatio, float testAverage) {
        float ssample = (float) (STDEV / Math.sqrt((double) n));
        float t = (averageRatio - testAverage) / ssample;
        return Math.abs(t);
    }

    public float getProteinLogPvalue(String proteinName, int sides, float alpha, double testvalue) {
        try {
            int length = ProtsAndPeps.get(proteinName).size();
            double[] observed = new double[length];
            int counter = 0;
            for (QuantifiedAminoAcidSequence QA : ProtsAndPeps.get(proteinName)) {
                if (QA.isChecked()) {
                    observed[counter] = (double) QA.getLogRatio();
                    counter++;

                }






            }
            double pVal = TestUtils.tTest(testvalue, observed);
            // System.out.println(observed.length);
            if (counter > 1) {
                return (float) pVal;
            } else {
                return 1.0f;
            }

        } catch (IllegalArgumentException ex) {
            //    Logger.getLogger("global").log(Level.SEVERE, null, ex);
            // System.out.println(ex.getMessage());
            return 1.0f;
        } catch (MathException ex) {
            //    Logger.getLogger("global").log(Level.SEVERE, null, ex);
            //   System.out.println(ex.getMessage());
            return 1.0f;
        }

    }

    public float getProteinPvalue(String proteinName, int sides, float alpha, double testvalue) {
        try {
            int length = ProtsAndPeps.get(proteinName).size();
            double[] observed = new double[length];
            int counter = 0;
            for (QuantifiedAminoAcidSequence QA : ProtsAndPeps.get(proteinName)) {
                if (QA.isChecked()) {
                    observed[counter] = (double) QA.getRatio();
                    counter++;

                }






            }
            double pVal = TestUtils.tTest(testvalue, observed);
            // System.out.println(observed.length);
            if (counter > 1) {
                return (float) pVal;
            } else {
                return 1.0f;
            }

        } catch (IllegalArgumentException ex) {
            //    Logger.getLogger("global").log(Level.SEVERE, null, ex);
            // System.out.println(ex.getMessage());
            return 1.0f;
        } catch (MathException ex) {
            //    Logger.getLogger("global").log(Level.SEVERE, null, ex);
            //   System.out.println(ex.getMessage());
            return 1.0f;
        }

    }

    public boolean isPeptideChecked(QuantifiedAminoAcidSequence QA) {
        return QA.isChecked();
    }

    private float getMedianRatio() {
        float totalRatio = 0f;
        float totalCount = 0;
        Iterator it = ProtsAndPeps.keySet().iterator();
        while (it.hasNext()) {
            float x = 0f;
            int count = 0;
            for (QuantifiedAminoAcidSequence QA : ProtsAndPeps.get(it.next())) {
                if (QA.isChecked()) {
                    x += QA.getRatio();
                    count++;

                }






            }
            totalRatio += (x / count);
            totalCount++;

        }






        return totalRatio / totalCount;
    }

    private float getMedianLogRatio() {
        float totalRatio = 0f;
        float totalCount = 0;
        ArrayList<Float> FloatArray = new ArrayList<Float>();
        Iterator it = ProtsAndPeps.keySet().iterator();
        while (it.hasNext()) {
            boolean hasvalues = false;
            float x = 0f;
            int count = 0;
            for (QuantifiedAminoAcidSequence QA : ProtsAndPeps.get(it.next())) {
                if (QA.isChecked()) {
                    x += QA.getLogRatio();
                    count++;

                    FloatArray.add(QA.getLogRatio());
                    hasvalues =
                            true;
                }

            }
        }
        /** find the median */
        Float[] FLArray = FloatArray.toArray(new Float[0]);
        Arrays.sort(FLArray);
        int index = FLArray.length / 2;
        System.out.println(index + " " + FLArray.length);
        /** even or odd */
        if (FLArray.length % 2 != 0) {
            /** odd numbers in array */
            //System.out.println("ODD -> "+ FLArray[index - 1]);
            // get the i-th element
            return FLArray[index];

        } else { // even numbers in array
            //  System.out.println("EVEN -> "+ FLArray[index - 1]+ " "+FLArray[index]);
            return ((FLArray[index - 1] + FLArray[index]) / 2);
        }

    }

    // headerIndex returns the column number of a header name
    public int headerIndex(String headerName) {
        String[] index = headers.get(1);
        int found = 0;
        for (int t = 0; t <
                index.length; t++) {
            if (index[t].equals(headerName)) {
                found = t;
            }

        }
        return found;
    }

    public float normalizeRatiosAgainstMedian() {
        float median = getMedianRatio();
        Iterator it = ProtsAndPeps.keySet().iterator();
        while (it.hasNext()) {
            float x = 0f;
            int count = 0;
            for (QuantifiedAminoAcidSequence QA : ProtsAndPeps.get(it.next())) {
                if (QA.isChecked()) {
                    x = QA.getRatio() / median;
                    QA.setRatio(x);
                }

            }
        }
        return median;
    }

    public float normalizeLogRatiosAgainstMedian() {
        float median = getMedianLogRatio();
        Iterator it = ProtsAndPeps.keySet().iterator();
        while (it.hasNext()) {
            float x = 0f;
            int count = 0;
            for (QuantifiedAminoAcidSequence QA : ProtsAndPeps.get(it.next())) {
                //    if (QA.isChecked()) {
                x = QA.getLogRatio() - median;
                QA.setLogRatio(x);
                //    }
            }

        }
        return median;
    }

    public float normalizeRatiosAgainstFactor(float factor) {
        //float median = getMedianRatio();
        Iterator it = ProtsAndPeps.keySet().iterator();
        while (it.hasNext()) {
            float x = 0f;
            int count = 0;
            for (QuantifiedAminoAcidSequence QA : ProtsAndPeps.get(it.next())) {
                if (QA.isChecked()) {
                    x = QA.getRatio() / factor;
                    QA.setRatio(x);
                }

            }
        }
        return factor;
    }

    public float normalizeLogRatiosAgainstFactor(float factor) {
        //float median = getMedianRatio();
        Iterator it = ProtsAndPeps.keySet().iterator();
        while (it.hasNext()) {
            float x = 0f;
            int count = 0;
            for (QuantifiedAminoAcidSequence QA : ProtsAndPeps.get(it.next())) {
                if (QA.isChecked()) {
                    x = QA.getLogRatio() - factor;
                    QA.setLogRatio(x);
                }

            }
        }
        return factor;
    }

    public Float[] getMAvalues(String protID) {
        Float[] MA = new Float[2];
        double V1 = 0;
        double V2 = 0;
        for (QuantifiedAminoAcidSequence QA : ProtsAndPeps.get(protID)) {
            if (QA.isChecked()) {
                V1 += Double.parseDouble(QA.getPeptideinfo()[peptideSignalA]);
                V2 +=
                        Double.parseDouble(QA.getPeptideinfo()[peptideSignalB]);
            }

        }
        // System.out.println(protID+"  "+V1+" "+V2);
        V1 = Math.log10(V1) / Math.log10(2);
        V2 =
                Math.log10(V2) / Math.log10(2);


        MA[0] = (float) ((V1 + V2) / 2);
        MA[1] = (float) (Math.log10(getProteinRatio(protID)) / Math.log10(2));
        // System.out.println(V1+" "+V2+" "+MA[0]+" "+MA[1]);
        if (MA[0].isNaN() || MA[1].isNaN()) {
            MA[0] = 0f;
            MA[1] = 0f;
            return MA;
        } else {
            return MA;
        }

    }

    /**
     *  This function checks if a peptide has been found in more than one protein.
     *  If this peptide was not unique it will become un/selected (checked) in the GUI, depending on its previous state. 
     *  Therefore the user can switch this selection on or off
     *  @boolean = true to enable this peptide again, false to uncheck it.
     */
    public void selectOrUnselectUniquePeptides(boolean b) {
        for (String protein : ProtsAndPeps.keySet()) {
            for (QuantifiedAminoAcidSequence seq : ProtsAndPeps.get(protein)) {
                if (!seq.isPeptideUnique()) {
                    seq.setChecked(b);
                }

            }
        }
    }

    /**
     *  This function performs a simple outlier detection based on Z-score
     *  The formula: (abs(x- mean) / std) > 3 
     *  Basically this looks if a datapoint exceeds the 3 STD threshold. If so it is considered an outlier
     *  An other methods would be a Q-score, however (not implemented) this only can discard of ONE outlier per set and
     *  should not be repeated more than once (because it will throw away all datapoints.
     *  This algorithm will take the values per protein, compare and set the peptide setChecked to false is z > 3STD
     *  I have adjusted this algoritm. It will leave out the value of the peptide that you are researching at that moment,
     *  recalulate the STDEV, and Mean, and see if your peptide is an outlier.
     *  Need at least 4 peptides per protein! (at least that is what I think.
     * @return
     */
    public int doOutlierDetection() {
        int totalcounter = 0;

        for (String protein : ProtsAndPeps.keySet()) {
            if (getProteinPeptideNumber(protein) > 3) { // We only apply outlier detection if more than 3 peptides are involved.
                ArrayList<Integer> Outliers = new ArrayList<Integer>(); // store all outliers in here by their array index number
                float oldSTDEV = getProteinLogSTDEV(protein);
                int counter = 0;
                for (QuantifiedAminoAcidSequence seq : ProtsAndPeps.get(protein)) {
                    // STart by setting the peptide to false (if not already false!)

                    if (seq.isChecked()) {
                        seq.setChecked(false);
                        // calculate the mean and STDEV
                        float mean = getProteinLogRatio(protein);
                        float STDEV = getProteinLogSTDEV(protein);
                        float Zscore = Math.abs(seq.getLogRatio() - mean) / STDEV;
                        // System.out.println("test outlier detection on  "+protein+" "+mean+" "+STDEV+" zscore "+Zscore);
                        if (Zscore >= 3f) { // peptide is an outlier.
                            Outliers.add(counter);
                        }

                        seq.setChecked(true);

                    }

                    counter++;
                } // end for loop
// Here we will uncheck all outliers

                for (Integer t : Outliers) {
                    ProtsAndPeps.get(protein).get(t).setChecked(false);
                    totalcounter++;

                }






            }
        }
        return totalcounter;
    }

    /**
     * Calculate the Arg -> Pro conversion in a sample based on the Non proline containing peptides
     *
     */
    public void findConversionFactorPerProtein() {
    }

    public boolean isFilteredForUniqueness_show() {
        return filteredForUniqueness_show;
    }

    public void setFilteredForUniqueness_show(boolean filteredForUniqueness_show) {
        this.filteredForUniqueness_show = filteredForUniqueness_show;
    }

    public boolean isFilteredForUniqueness() {
        return filteredForUniqueness;
    }

    public void setFilteredForUniqueness(boolean filteredForUniqueness) {
        this.filteredForUniqueness = filteredForUniqueness;
    }

    public int getQueryColNumber() {
        return QueryColNumber;
    }

    public void setQueryColNumber(int QueryColNumber) {
        this.QueryColNumber = QueryColNumber;
    }

    public int getPepScoreColNumber() {
        return pepScoreColNumber;
    }

    public void setPepScoreColNumber(int pepScoreColNumber) {
        this.pepScoreColNumber = pepScoreColNumber;
    }

    public boolean isUsePeptideReport() {
        return usePeptideReport;
    }

    public void setUsePeptideReport(boolean usePeptideReport) {
        this.usePeptideReport = usePeptideReport;
    }

    public int getPTMpositionColNumber() {
        return PTMpositionColNumber;
    }

    public void setPTMpositionColNumber(int PTMpositionColNumber) {
        this.PTMpositionColNumber = PTMpositionColNumber;
    }
// GETTERS AND SETTERS AREA

    public boolean isUseUniquePeptidesOnly() {

        return useUniquePeptidesOnly;
    }

    public void setUseUniquePeptidesOnly(boolean useUniquePeptidesOnly) {
        this.useUniquePeptidesOnly = useUniquePeptidesOnly;
    }

    public int getProtDescriptionColumn() {
        return protDescriptionColumn;
    }

    public void setProtDescriptionColumn(int protDescriptionColumn) {
        this.protDescriptionColumn = protDescriptionColumn;
    }

    public String[] getGenericHeader() {
        return headers.get(1);
    }

    public String[] getMSQuantHeader() {
        return headers.get(1);
    }

    public int getModificationColumn() {
        return modificationColumn;
    }

    public void setModificationColumn(int modificationColumn) {
        this.modificationColumn = modificationColumn;
    }

    public int getPeptideColumn() {
        return peptideColumn;
    }

    public int getPeptideSignalA() {
        return peptideSignalA;
    }

    public int getPeptideSignalB() {
        return peptideSignalB;
    }

    public int getProteinNameColumn() {
        return proteinNameColumn;
    }

    public void setFileName(String fileName) {
        this.fileName = fileName;
    }

    public void setPeptideColumn(int peptideColumn) {
        this.peptideColumn = peptideColumn;
    }

    public void setPeptideSignalA(int peptideSignalA) {
        this.peptideSignalA = peptideSignalA;
    }

    public void setPeptideSignalB(int peptideSignalB) {
        this.peptideSignalB = peptideSignalB;
    }

    public void setProteinNameColumn(int proteinNameColumn) {
        this.proteinNameColumn = proteinNameColumn;
    }

    public void setUseParserName(String useParserName) {
        this.useParserName = useParserName;
    }

    public void setUseParserVersion(float useParserVersion) {
        this.useParserVersion = useParserVersion;
    }

    public HashMap<String, String[]> getProteinInfo() {
        return ProteinInfo;
    }

    public void setProteinInfo(HashMap<String, String[]> ProteinInfo) {
        this.ProteinInfo = ProteinInfo;
    }

    public HashMap<String, ArrayList<QuantifiedAminoAcidSequence>> getProtsAndPeps() {
        return ProtsAndPeps;
    }

    public void setProtsAndPeps(HashMap<String, ArrayList<QuantifiedAminoAcidSequence>> ProtsAndPeps) {
        this.ProtsAndPeps = ProtsAndPeps;
    }

    public boolean isIsPConverted() {
        return isPConverted;
    }

    public void setIsPConverted(boolean isPConverted) {
        this.isPConverted = isPConverted;
    }

    public float getConversionFactor() {
        return conversionFactor;
    }

    public void setConversionFactor(float conversionFactor) {
        this.conversionFactor = conversionFactor;
    }

    public int getScanNumberColNumber() {
        return ScanNumberColNumber;
    }

    public void setScanNumberColNumber(int ScanNumberColNumber) {
        this.ScanNumberColNumber = ScanNumberColNumber;
    }

    public ProtsAndPepsDataHandler() {
    }

    public ProtsAndPepsDataHandler(HashMap<String, ArrayList<QuantifiedAminoAcidSequence>> PAP, HashMap<String, String[]> Pinfo) {
        ProteinInfo = Pinfo;
        ProtsAndPeps =
                PAP;
    }
}
