package nl.uu.chem.bioinformatics.statquant.gui;

import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.Iterator;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableColumn;
import javax.swing.table.TableRowSorter;
import nl.uu.chem.bioinformatics.statquant.ProtsAndPepsDataHandler;
import nl.uu.chem.bioinformatics.statquant.model.QuantifiedAminoAcidSequence;
import nl.uu.chem.bioinformatics.statquant.util.qvalue;
import org.jdesktop.layout.GroupLayout;

class ExperimentPanel extends JPanel {

    private ProtsAndPepsDataHandler protsAndPepsObject = new ProtsAndPepsDataHandler();
    private JTable proteinComparisonTable = new JTable();
    private JScrollPane scrollPane = new JScrollPane();
    private DefaultTableModel model = new DefaultTableModel();
    private JTable bottomTable;
    private JTabbedPane tabbedPane;
    private JDialog correctionDialog;
    private JTextField JTF;
    private String protID;

    public ExperimentPanel(ProtsAndPepsDataHandler protsAndPepsDataHandlerObject, JTable bottomTable, JTabbedPane JTP, JDialog corrDialog, JTextField corrfactor) {
        super();
        this.bottomTable = bottomTable;
        correctionDialog = corrDialog;
        protsAndPepsObject = protsAndPepsDataHandlerObject;
        tabbedPane = JTP;
        JTF = corrfactor;
        makeTable();
        initialize();
    }

    private void initialize() {
        scrollPane.setViewportView(proteinComparisonTable);
        proteinComparisonTable.addKeyListener(new KeyListener() {

            @Override
            public void keyTyped(KeyEvent e) {
            }

            @Override
            public void keyPressed(KeyEvent e) {
                jTable1KeyTyped(e);
            }

            @Override
            public void keyReleased(KeyEvent e) {
            }
        });
        proteinComparisonTable.addMouseListener(new MouseAdapter() {

            @Override
            public void mouseReleased(MouseEvent evt) {
                if (evt.getButton() == 1) {
                    jTable1MouseReleased(evt);
                } else {
                    jTable1RightMouseReleased(evt);
                }
            }
        });
        GroupLayout PanelLayout = new GroupLayout(this);
        setLayout(PanelLayout);
        PanelLayout.setHorizontalGroup(PanelLayout.createParallelGroup(GroupLayout.LEADING).add(scrollPane, GroupLayout.DEFAULT_SIZE, 739, Short.MAX_VALUE));
        PanelLayout.setVerticalGroup(PanelLayout.createParallelGroup(GroupLayout.LEADING).add(scrollPane, GroupLayout.DEFAULT_SIZE, 337, Short.MAX_VALUE));
    }

    public DefaultTableModel getDefaultTableModel() {
        return model;
    }

    @SuppressWarnings(value = "unchecked")
    public void setDTM(DefaultTableModel DTM) {
        this.model = DTM;
        proteinComparisonTable.setModel(DTM);
        TableRowSorter sorter = new TableRowSorter(DTM);
        sorter.setComparator(0, new Comparator<Integer>() {

            @Override
            public int compare(Integer o1, Integer o2) {
                return o1.compareTo(o2);
            }
        });
        sorter.setComparator(5, new Comparator<Integer>() {

            @Override
            public int compare(Integer o1, Integer o2) {
                return o1.compareTo(o2);
            }
        });
        sorter.setComparator(2, new Comparator<Float>() {

            @Override
            public int compare(Float o1, Float o2) {
                return o1.compareTo(o2);
            }
        });
        sorter.setComparator(3, new Comparator<Float>() {

            @Override
            public int compare(Float o1, Float o2) {
                return o1.compareTo(o2);
            }
        });
        sorter.setComparator(4, new Comparator<Float>() {

            @Override
            public int compare(Float o1, Float o2) {
                return o1.compareTo(o2);
            }
        });
        sorter.setComparator(7, new Comparator<Double>() {

            @Override
            public int compare(Double o1, Double o2) {
                return o1.compareTo(o2);
            }
        });
        sorter.setComparator(6, new Comparator<Float>() {

            @Override
            public int compare(Float o1, Float o2) {
                return o1.compareTo(o2);
            }
        });
        proteinComparisonTable.setRowSorter(sorter);
    }

    public ProtsAndPepsDataHandler getPPDH() {
        return protsAndPepsObject;
    }

    public void setPPDH(ProtsAndPepsDataHandler PPDH) {
        this.protsAndPepsObject = PPDH;
    }

    @SuppressWarnings(value = "unchecked")
    public void makeTable() {
        ArrayList<Integer> number = new ArrayList<Integer>();
        ArrayList<Float> ratios = new ArrayList<Float>();
        ArrayList<Float> stdevs = new ArrayList<Float>();
        ArrayList<Integer> nrpeptides = new ArrayList<Integer>();
        ArrayList<Float> tstats = new ArrayList<Float>();
        ArrayList<Float> Pvalues = new ArrayList<Float>();
        ArrayList<Float> logRatios = new ArrayList<Float>();
        Iterator<String> it = protsAndPepsObject.getProtsAndPeps().keySet().iterator();
        ArrayList<String> descriptions = new ArrayList<String>();
        int counter = 1;
        while (it.hasNext()) {
            String Prot = it.next();
            float ratio = protsAndPepsObject.getProteinRatio(Prot);
            ratios.add(ratio);
            float logratio = (float) (Math.log10(ratio) / Math.log10(2));
            //logRatios.add(logratio);
            logRatios.add(protsAndPepsObject.getProteinLogRatio(Prot));
            stdevs.add(protsAndPepsObject.getProteinLogSTDEV(Prot));
            nrpeptides.add(protsAndPepsObject.getProteinPeptideNumber(Prot));
            //   tstats.add(experiments.get(experimentNumbers).getProteinTstatistics(experiments.get(experimentNumbers).getProteinSTDEV(Prot), experiments.get(experimentNumbers).getProteinPeptideNumber(Prot), experiments.get(experimentNumbers).getProteinRatio(Prot), 1f));
            Pvalues.add(protsAndPepsObject.getProteinLogPvalue(Prot, 2, 0.5F, 0.0));
            descriptions.add(protsAndPepsObject.getProteinDescription(Prot));
            number.add(counter);
            counter++;
        }
        // BUILD THE TABLE MODEL
        model.addColumn("#", number.toArray());
        model.addColumn("protID", protsAndPepsObject.getProtsAndPeps().keySet().toArray());
        model.addColumn("Ratio", ratios.toArray());
        model.addColumn("log Ratio", logRatios.toArray());
        model.addColumn("(log) STDEV", stdevs.toArray());
        model.addColumn("nr peptides", nrpeptides.toArray());
        //  DTM.addColumn("T statistics", tstats.toArray());
        model.addColumn("P-value", Pvalues.toArray());
        ArrayList<Double> Qvalues = new ArrayList<Double>();
        qvalue qval = new qvalue(Pvalues.toArray(), Pvalues.size());
        double[] q1 = qval.q_values();
        for (int t = 0; t < q1.length; t++) {
            Qvalues.add(q1[t]);
        }
        model.addColumn("Qvalue", Qvalues.toArray());
        model.addColumn("Description", descriptions.toArray());
        proteinComparisonTable.setModel(model);
        proteinComparisonTable.setAutoCreateRowSorter(true);
        TableRowSorter sorter = new TableRowSorter(model);
        sorter.setComparator(0, new Comparator<Integer>() {

            @Override
            public int compare(Integer o1, Integer o2) {
                return o1.compareTo(o2);
            }
        });
        sorter.setComparator(5, new Comparator<Integer>() {

            @Override
            public int compare(Integer o1, Integer o2) {
                return o1.compareTo(o2);
            }
        });
        sorter.setComparator(2, new Comparator<Float>() {

            @Override
            public int compare(Float o1, Float o2) {
                return o1.compareTo(o2);
            }
        });
        sorter.setComparator(3, new Comparator<Float>() {

            @Override
            public int compare(Float o1, Float o2) {
                return o1.compareTo(o2);
            }
        });
        sorter.setComparator(4, new Comparator<Float>() {

            @Override
            public int compare(Float o1, Float o2) {
                return o1.compareTo(o2);
            }
        });
        sorter.setComparator(7, new Comparator<Double>() {

            @Override
            public int compare(Double o1, Double o2) {
                return o1.compareTo(o2);
            }
        });
        sorter.setComparator(6, new Comparator<Float>() {

            @Override
            public int compare(Float o1, Float o2) {
                return o1.compareTo(o2);
            }
        });
        proteinComparisonTable.setRowSorter(sorter);
        // Table Markup
        int vColIndex = 0;
        TableColumn col = proteinComparisonTable.getColumnModel().getColumn(vColIndex);
        int width = 50;
        col.setPreferredWidth(width);
        vColIndex = 3;
        col = proteinComparisonTable.getColumnModel().getColumn(vColIndex);
        col.setPreferredWidth(width);
    }

    private void jTable1KeyTyped(KeyEvent key) {
        if (key.isActionKey()) {
            int row = proteinComparisonTable.getSelectionModel().getLeadSelectionIndex();
            if (key.getKeyCode() == KeyEvent.VK_DOWN) {
                row = row + 1;
            }
            if (key.getKeyCode() == KeyEvent.VK_UP) {
                row = row - 1;
            }
            int column = proteinComparisonTable.getColumnModel().getSelectionModel().getLeadSelectionIndex();
            protID = (String) proteinComparisonTable.getValueAt(row, 1);
            buildBottomTable(protID);
        }
    }

    @SuppressWarnings(value = "unchecked")
    private void buildBottomTable(String proteinName) {
        DefaultTableModel pepDTM = new DefaultTableModel() {

            @Override
            public Class getColumnClass(int columnIndex) {
                if (columnIndex == 1) {
                    return Boolean.class;
                } else {
                    return super.getColumnClass(columnIndex);
                }
            }
        };
        ArrayList<String> peptides = new ArrayList<String>();
        ArrayList<Float> ratios = new ArrayList<Float>();
        ArrayList<Integer> number = new ArrayList<Integer>();
        ArrayList<String> mods = new ArrayList<String>();
        ArrayList<Integer> nrProlines = new ArrayList<Integer>();
        ArrayList<Float> oldRatio = new ArrayList<Float>();
        ArrayList<Boolean> checked = new ArrayList<Boolean>();
        ArrayList<String> checkedPTM = new ArrayList<String>();
        ArrayList<String> Scannumber = new ArrayList<String>();
        ArrayList<String> uniqueness = new ArrayList<String>();
        ArrayList<String> QueryNR = new ArrayList<String>();
        ArrayList<String> Score = new ArrayList<String>();
        ArrayList<String> Intensity1 = new ArrayList<String>();
        ArrayList<String> Intensity2 = new ArrayList<String>();
        int count = 1;
        for (QuantifiedAminoAcidSequence QA : protsAndPepsObject.getProtsAndPeps().get(protID)) {
            number.add(count);
            peptides.add(QA.getSequence());
            ratios.add(QA.getRatio());
            checked.add(QA.isChecked());
            // if (PPDH.isUsePeptideReport()){ //PTMchecked INFO is used
            //    checkedPTM.add(QA.getSeqWithMods());
            //}
            if (protsAndPepsObject.getPepScoreColNumber() > 0) {
                Score.add(QA.getPeptideinfo()[protsAndPepsObject.getPepScoreColNumber()]);
            }
            if (protsAndPepsObject.getQueryColNumber() > 0) {
                QueryNR.add(QA.getPeptideinfo()[protsAndPepsObject.getQueryColNumber()]);
            }
            if (protsAndPepsObject.isFilteredForUniqueness()) {
                uniqueness.add(QA.getPresentInProteins().toString());
            }
            if (protsAndPepsObject.getPTMpositionColNumber() > 0) {
                checkedPTM.add(QA.getSeqWithMods());
            }
            if (protsAndPepsObject.getScanNumberColNumber() > 0) {
                Scannumber.add(QA.getPeptideinfo()[protsAndPepsObject.getScanNumberColNumber()]);
            }
            // get the modifications for that peptide
            if (protsAndPepsObject.isIsPConverted()) {
                // show additional info in peptide table
                nrProlines.add(protsAndPepsObject.getAminoAcidOccurence("P", QA.getSequence()));
                oldRatio.add(protsAndPepsObject.prolineConversion(QA.getSequence(), protsAndPepsObject.getConversionFactor()));
            }
            Intensity1.add(QA.getPeptideinfo()[protsAndPepsObject.getPeptideSignalA()]);
            Intensity2.add(QA.getPeptideinfo()[protsAndPepsObject.getPeptideSignalB()]);
            mods.add(protsAndPepsObject.getPeptideModification(protID, count - 1));
            count++;
        }
        pepDTM.addColumn("#", number.toArray());
        pepDTM.addColumn("include", checked.toArray());
        pepDTM.addColumn("Peptides", peptides.toArray());
        pepDTM.addColumn("modification(s)", mods.toArray());
        pepDTM.addColumn("Ratio", ratios.toArray());
        if (protsAndPepsObject.isIsPConverted()) {
            // show additional info in peptide table
            pepDTM.addColumn("# prolines", nrProlines.toArray());
            pepDTM.addColumn("corr. factor", oldRatio.toArray());
        }
        if (protsAndPepsObject.getPTMpositionColNumber() > 0) {
            pepDTM.addColumn("Checked PTM", checkedPTM.toArray());
        }
        if (protsAndPepsObject.getPepScoreColNumber() > 0) {
            pepDTM.addColumn("peptide score", Score.toArray());
        }
        if (protsAndPepsObject.getQueryColNumber() > 0) {
            pepDTM.addColumn("Query #", QueryNR.toArray());
        }
        if (protsAndPepsObject.getScanNumberColNumber() > 0) {
            pepDTM.addColumn("ScanNumber", Scannumber.toArray());
        }
        if (protsAndPepsObject.isFilteredForUniqueness()) {
            pepDTM.addColumn("peptide present in", uniqueness.toArray());
        }
        pepDTM.addColumn("Signal A", Intensity1.toArray());
        pepDTM.addColumn("Signal B", Intensity2.toArray());
        bottomTable.setModel(pepDTM);
        bottomTable.setAutoCreateRowSorter(true);
        TableRowSorter sorter = new TableRowSorter(pepDTM);
        sorter.setComparator(0, new Comparator<Integer>() {

            @Override
            public int compare(Integer o1, Integer o2) {
                return o1.compareTo(o2);
            }
        });
        sorter.setComparator(4, new Comparator<Float>() {

            @Override
            public int compare(Float o1, Float o2) {
                return o1.compareTo(o2);
            }
        });
        bottomTable.setRowSorter(sorter);
        pepDTM.addTableModelListener(new TableModelListener() {

            @Override
            public void tableChanged(TableModelEvent e) {
                //throw new UnsupportedOperationException("Not supported yet.");
                int row = bottomTable.getSelectionModel().getLeadSelectionIndex();
                int column = bottomTable.getColumnModel().getSelectionModel().getLeadSelectionIndex();
                if (column == 1) {
                    // peptide number =
                    int peptideNumber = Integer.parseInt(bottomTable.getValueAt(row, 0).toString()) - 1;
                    //  System.out.println(PPDH.getProtsAndPeps().get(protID).get(peptideNumber).getSequence() + " is "+ bottomTable.getValueAt(row, 1));
                    protsAndPepsObject.getProtsAndPeps().get(protID).get(peptideNumber).setChecked((Boolean) bottomTable.getValueAt(row, 1));
                    model = new DefaultTableModel();
                    makeTable();
                }
            }
        });
        tabbedPane.setTitleAt(0, protID);
    }

    private void jTable1MouseReleased(MouseEvent evt) {
        //System.out.println("EVENT DETECTED !");
        int row = proteinComparisonTable.getSelectionModel().getLeadSelectionIndex();
        int column = proteinComparisonTable.getColumnModel().getSelectionModel().getLeadSelectionIndex();
        protID = (String) proteinComparisonTable.getValueAt(row, 1);
        /**    int[] selectedRows = proteinComparisonTable.getSelectedRows();
        String ii ="";
        for (int i=0; i <selectedRows.length ; i++){
        ii = ii + ", "+selectedRows[i];
        }
        System.out.println(ii); */
        buildBottomTable(protID);
    }

    private void jTable1RightMouseReleased(MouseEvent evt) {
        int row = proteinComparisonTable.getSelectionModel().getLeadSelectionIndex();
        int column = proteinComparisonTable.getColumnModel().getSelectionModel().getLeadSelectionIndex();
        int[] selectedRows = proteinComparisonTable.getSelectedRows();
        String ii = "";
        Float aveRatio = 0.0F;
        int count = 0;
        for (int i = 0; i < selectedRows.length; i++) {
            String ProtID = (String) proteinComparisonTable.getValueAt(selectedRows[i], 1);
            float ratio = protsAndPepsObject.getProteinLogRatio(ProtID);
            aveRatio = aveRatio + ratio;
            count++;
        }
        aveRatio = aveRatio / count;
        JTF.setText(aveRatio.toString());
        correctionDialog.setVisible(true);
        correctionDialog.pack();
    }
}
