/*
 * MSQuantParser.java
 *
 * Created on Jul 30, 2007, 11:54:28 AM
 *
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package nl.uu.chem.bioinformatics.statquant.parsers;

import nl.uu.chem.bioinformatics.statquant.model.QuantifiedAminoAcidSequence;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.regex.Pattern;
import java.util.regex.Matcher;
import nl.uu.chem.bioinformatics.statquant.util.RandomAccessFile;


/**
 *
 * @author breuk102
 */
public class MSQuantParser {

    // Generic info to be used when I want to implement a kind of plug and play thing in my program
    private String ParserName = "MSQuantParser";
    private float ParserVersion = 0.1f;

    private String filename;
    private String Protein;
    private HashMap<String, String[]> ProteinInfo = new HashMap<String, String[]>();
    private QuantifiedAminoAcidSequence QAAS;
    private HashMap<String, ArrayList<QuantifiedAminoAcidSequence>> ProtsAndPeps = new HashMap<String, ArrayList<QuantifiedAminoAcidSequence>>();
    private RandomAccessFile RF;
    private File MSQfile;
    private boolean headerIsRead = false;
    private boolean useUniquePeptidesOnly = false; // Is true ADD peptide areas of not unique peptides from the same protein
    private boolean usePeptideReport = false; // Is true than protein names are not hashes for ProtsAndPeps but the sequences;
    
    // An absolutely required setting... which column contains the protein name!
    private int proteinNameColumn = 0; //start counting from 0 for column 1
    // Other required settings are the columns for signals en peptideseq.
    private int peptideColumn = 0; //starting from 0!
    private int peptideSignalA = 0; //actually this is not used in this parser. done by datahandler
    private int peptideSignalB = 0; // idem. (only used in debuggung for now)
    private int proteinDescriptionColumn = 0;
    private int PTMpositionColNumber=0;
    
    
    
    private boolean setColumns = true;
    // MSQUANT SPECIFIC?
    private int modificationColumn = 36;

    public int getModificationColumn() {
        return modificationColumn;
    }

    public void setModificationColumn(int modificationColumn) {
        this.modificationColumn = modificationColumn;
    }

    public boolean isUsePeptideReport() {
        return usePeptideReport;
    }

    public void setUsePeptideReport(boolean usePeptideReport) {
        this.usePeptideReport = usePeptideReport;
    }
    
    
    
    
    private ArrayList<String[]> headers = new ArrayList<String[]>();

    public MSQuantParser() {
    }



    public void run() {
        try {
            MSQfile = new java.io.File(filename);
            readFile();
        } catch (IOException ex) {
            Logger.getLogger("global").log(Level.SEVERE, null, ex);
        }
        getProtsAndPeps();
    }

    public int getPTMpositionColNumber() {
        return PTMpositionColNumber;
    }

    public void setPTMpositionColNumber(int PTMpositionColNumber) {
        this.PTMpositionColNumber = PTMpositionColNumber;
    }

    
    public int getProteinDescriptionColumn() {
        return proteinDescriptionColumn;
    }

    public void setProteinDescriptionColumn(int proteinDescriptionColumn) {
        this.proteinDescriptionColumn = proteinDescriptionColumn;
    }

    
    
    
    public void setProteinInfo(HashMap<String, String[]> ProteinInfo) {
        this.ProteinInfo = ProteinInfo;
    }

    public HashMap<String, String[]> getProteinInfo() {
        return ProteinInfo;
    }

    public void setPeptideColumn(int peptideColumn) {
        this.peptideColumn = peptideColumn;
    }

    public void setPeptideSignalA(int peptideSignalA) {
        this.peptideSignalA = peptideSignalA;
    }

    public void setPeptideSignalB(int peptideSignalB) {
        this.peptideSignalB = peptideSignalB;
    }

    public void setProteinNameColumn(int c) {
        this.proteinNameColumn = c;
    }

    public int getPeptideColumn() {
        return peptideColumn;
    }

    public int getPeptideSignalA() {
        return peptideSignalA;
    }

    public int getPeptideSignalB() {
        return peptideSignalB;
    }

    public int getProteinNameColumn() {
        return proteinNameColumn;
    }



    public void setFileName(String fn) {
        this.filename = fn;
    }

    public String getParserName() {
        return ParserName;
    }

    public float getParserVersion() {
        return ParserVersion;
    }

    public void setProtsAndPeps(HashMap<String, ArrayList<QuantifiedAminoAcidSequence>> ProtsAndPeps) {
        this.ProtsAndPeps = ProtsAndPeps;
    }

    public HashMap getProtsAndPeps() {

        return ProtsAndPeps;
    }

    // This getHeaders functions returns all headers found as defined in the parser. A generic parser would return more or less headers.
    public ArrayList<String[]> getHeaders() {

        return headers;
    }

    public boolean isUseUniquePeptidesOnly() {
        return useUniquePeptidesOnly;
    }

    public void setUseUniquePeptidesOnly(boolean useUniquePeptidesOnly) {
        this.useUniquePeptidesOnly = useUniquePeptidesOnly;
    }





    public int headerIndex(String headerName) {
        String[] index = this.headers.get(1);
        int found = 0;
        for (int t = 0; t < index.length; t++) {
            if (index[t].equals(headerName)) {
                found = t;
            }
        }
        return found;
    }

    private void updateColumnInfo() {
        if (this.setColumns) {
            System.out.println("Finding columns...");
            this.peptideColumn = this.headerIndex("Sequence");
            this.modificationColumn = this.headerIndex("Modifications");
            this.peptideSignalA = this.headerIndex("Intensity 1");
            this.peptideSignalB = this.headerIndex("Intensity 2");
            System.out.println(this.peptideColumn + " " + this.modificationColumn + " " + this.peptideSignalA + " " + this.peptideSignalB);
        }
    }


    public void determineColumnsFromFirstRead() throws IOException {
        String line;
        String protein = "";
        MSQfile = new java.io.File(filename);
        boolean Peplines = false;
        RF = new RandomAccessFile(MSQfile, "r");
        search:
        while ((line = RF.readLine()) != null) {
            line = line.trim();
            String[] Line_array = line.split("\t");

            Pattern pattern = Pattern.compile("^PROTEIN.*");
            Matcher matcher = pattern.matcher(line);
            if (matcher.matches()) {
                String hline = RF.readLine().trim(); //skip one line (this contains the peptide header!
                this.headers.add(0, hline.split("\t"));
                this.headers.add(1, hline.split("\t"));
                // System.out.println(hline);
                break search;
            }
        }
        // we have the header.. lets determine the actual header columns
        this.peptideColumn = this.headerIndex("Sequence");
        this.modificationColumn = this.headerIndex("Modifications");
        this.peptideSignalA = this.headerIndex("Intensity 1");
        this.peptideSignalB = this.headerIndex("Intensity 2");
    }



    private void readFile() throws IOException {
        String line;
        String protein = "";

        boolean Peplines = false;
        RF = new RandomAccessFile(MSQfile, "r");
        ArrayList<Object> RAFilePointers = new ArrayList<Object>();
        // RAFilePointers.add(RF.getFilePointer()); //start of doc. 1st line
        while ((line = RF.readLine()) != null) {
            RAFilePointers.add(RF.getFilePointer());
            line = line.trim();
            String[] Line_array = line.split("\t");

            Pattern pattern = Pattern.compile("^PROTEIN.*");
            Matcher matcher = pattern.matcher(line);
            if (matcher.matches()) {
                // get protein Header
                if (!(headerIsRead)) {
                    int RAFlength = RAFilePointers.size();
                    RF.seek(Long.parseLong(RAFilePointers.get(RAFlength - 3).toString())); // go to header line
                    String oldLine = RF.readLine();
                    oldLine = oldLine.trim();
                    String[] oldLine_array = oldLine.split("\t");
                    this.headers.add(0, oldLine_array);
                    RF.seek(Long.parseLong(RAFilePointers.get(RAFlength - 1).toString())); // go back to current line
                    headerIsRead = true;
                }

                protein = Line_array[this.proteinNameColumn];
                if (ProteinInfo.containsKey(protein)) {
                    // do nothing.
                } else {
                    // add new protein
                    ProteinInfo.put(protein, Line_array);
                }
                // skip one line, to obtain the peptide header info
                Peplines = false;
                String hline = RF.readLine().trim();
                this.headers.add(1, hline.split("\t"));
                Peplines = true;
            } else {
                if ((!(line.equalsIgnoreCase(""))) && Peplines) {

                    boolean pepOK = false;
                    float _sigA = Float.parseFloat(Line_array[this.peptideSignalA]);
                    float _sigB = Float.parseFloat(Line_array[this.peptideSignalB]);
                    if (_sigA > 0f && _sigB > 0f) {

                        QAAS = new QuantifiedAminoAcidSequence();
                        if (ProtsAndPeps.containsKey(protein)) {

                            // Check if peptide already exists for this protein.
                            if (this.useUniquePeptidesOnly) {
                                boolean unique = true;
                                uniquepeptide:
                                for (int t = 0; t < ProtsAndPeps.get(protein).size(); t++) {
                                    if (ProtsAndPeps.get(protein).get(t).getSequence().equals(Line_array[this.peptideColumn])) {
                                        // next make sure that it has the same modifications
                                        if (ProtsAndPeps.get(protein).get(t).getModification().equals(Line_array[this.modificationColumn])) {
                                            // ADD AREAS
                                            String[] pepinfo = ProtsAndPeps.get(protein).get(t).getPeptideinfo();
                                            float sigA = Float.parseFloat(Line_array[this.peptideSignalA]);
                                            float sigB = Float.parseFloat(Line_array[this.peptideSignalB]);
                                            sigA += Float.parseFloat(pepinfo[this.peptideSignalA]);
                                            sigB += Float.parseFloat(pepinfo[this.peptideSignalB]);
                                            pepinfo[this.peptideSignalA] = Float.toString(sigA);
                                            pepinfo[this.peptideSignalB] = Float.toString(sigB);
                                            ProtsAndPeps.get(protein).get(t).setPeptideinfo(pepinfo);
                                            float ratioBA = Float.parseFloat(pepinfo[this.peptideSignalB]) / Float.parseFloat(pepinfo[this.peptideSignalA]);
                                            ProtsAndPeps.get(protein).get(t).setRatio(ratioBA);
                                            unique = false;
                                            break uniquepeptide;
                                        }
                                    }
                                }
                                // if uniue !!
                                if (unique) {
                                    QAAS.setSequence(Line_array[this.peptideColumn]);
                                    QAAS.setPeptideinfo(Line_array);
                                    QAAS.setModification(Line_array[this.modificationColumn]);
                                    float ratioBA = Float.parseFloat(Line_array[this.peptideSignalB]) / Float.parseFloat(Line_array[this.peptideSignalA]);
                                    QAAS.setRatio(ratioBA);
                                    ProtsAndPeps.get(protein).add(QAAS);
                                }

                                // END UNIQUE*/
                            } else {
                                QAAS.setSequence(Line_array[this.peptideColumn]);
                                QAAS.setPeptideinfo(Line_array);
                                QAAS.setModification(Line_array[this.modificationColumn]);
                                float ratioBA = Float.parseFloat(Line_array[this.peptideSignalB]) / Float.parseFloat(Line_array[this.peptideSignalA]);
                                QAAS.setRatio(ratioBA);
                                ProtsAndPeps.get(protein).add(QAAS);
                            }
                        } else {
                            //new protein
                            QAAS.setSequence(Line_array[this.peptideColumn]);
                            QAAS.setPeptideinfo(Line_array);
                            QAAS.setModification(Line_array[this.modificationColumn]);
                            float ratioBA = Float.parseFloat(Line_array[this.peptideSignalB]) / Float.parseFloat(Line_array[this.peptideSignalA]);
                            QAAS.setRatio(ratioBA);
                            ArrayList<QuantifiedAminoAcidSequence> QAASlist = new ArrayList<QuantifiedAminoAcidSequence>();
                            QAASlist.add(QAAS);
                            ProtsAndPeps.put(protein, QAASlist);
                        }
                    }
                }
            }
        }
    }
}
