/*
 * MSQuantParser.java
 *
 * Created on Jul 30, 2007, 11:54:28 AM
 *
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package nl.uu.chem.bioinformatics.statquant.parsers;

import nl.uu.chem.bioinformatics.statquant.model.QuantifiedAminoAcidSequence;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.regex.Pattern;
import java.util.regex.Matcher;
import nl.uu.chem.bioinformatics.statquant.util.RandomAccessFile;

/**
 *
 * @author breuk102
 */
public class MSQuantParser2 {

    // Generic info to be used when I want to implement a kind of plug and play thing in my program
    private String ParserName = "MSQuantParser2"; // Parser Protein header less MSQuantOutputfiles
    private float ParserVersion = 0.1f;
    private String filename;
    private String Protein;
    private HashMap<String, String[]> ProteinInfo = new HashMap<String, String[]>();
    private QuantifiedAminoAcidSequence QAAS;
    private HashMap<String, ArrayList<QuantifiedAminoAcidSequence>> ProtsAndPeps = new HashMap<String, ArrayList<QuantifiedAminoAcidSequence>>();
    private RandomAccessFile RF;
    private File MSQfile;
    private HashMap<String, ArrayList<String>> mods = new HashMap<String, ArrayList<String>>();
    private HashMap<String, String> symbol = new HashMap<String, String>();
    private boolean headerIsRead = false;
    private boolean useUniquePeptidesOnly = false; // Is true ADD peptide areas of not unique peptides from the same protein
    private boolean usePeptideReport = false; // Is true than protein names are not hashes for ProtsAndPeps but the sequences;
    // An absolutely required setting... which column contains the protein name!
    private int proteinNameColumn = 0; //start counting from 0 for column 1
    // Other required settings are the columns for signals en peptideseq.
    private int peptideColumn = 0; //starting from 0!
    private int peptideSignalA = 0; //actually this is not used in this parser. done by datahandler
    private int peptideSignalB = 0; // idem. (only used in debuggung for now)
    private int proteinDescriptionColumn = 0;
    private int PTMpositionColNumber = 0;
    private boolean setColumns = true;
    // MSQUANT SPECIFIC?
    private int modificationColumn = 36;

    public int getModificationColumn() {
        return modificationColumn;
    }

    public void setModificationColumn(int modificationColumn) {
        this.modificationColumn = modificationColumn;
    }
    private ArrayList<String[]> headers = new ArrayList<String[]>();

    public void run() {
        try {
            MSQfile = new java.io.File(filename);
            readFile();
        } catch (IOException ex) {
            Logger.getLogger("global").log(Level.SEVERE, null, ex);
        }
        getProtsAndPeps();
    }

    public boolean isUsePeptideReport() {
        return usePeptideReport;
    }

    public void setUsePeptideReport(boolean usePeptideReport) {
        this.usePeptideReport = usePeptideReport;
    }

    public int getPTMpositionColNumber() {
        return PTMpositionColNumber;
    }

    public void setPTMpositionColNumber(int PTMpositionColNumber) {
        this.PTMpositionColNumber = PTMpositionColNumber;
    }

    public int getProteinDescriptionColumn() {
        return proteinDescriptionColumn;
    }

    public void setProteinDescriptionColumn(int proteinDescriptionColumn) {
        this.proteinDescriptionColumn = proteinDescriptionColumn;
    }

    public void setProteinInfo(HashMap<String, String[]> ProteinInfo) {
        this.ProteinInfo = ProteinInfo;
    }

    public HashMap<String, String[]> getProteinInfo() {
        return ProteinInfo;
    }

    public void setPeptideColumn(int peptideColumn) {
        this.peptideColumn = peptideColumn;
    }

    public void setPeptideSignalA(int peptideSignalA) {
        this.peptideSignalA = peptideSignalA;
    }

    public void setPeptideSignalB(int peptideSignalB) {
        this.peptideSignalB = peptideSignalB;
    }

    public void setProteinNameColumn(int c) {
        this.proteinNameColumn = c;
    }

    public int getPeptideColumn() {
        return peptideColumn;
    }

    public int getPeptideSignalA() {
        return peptideSignalA;
    }

    public int getPeptideSignalB() {
        return peptideSignalB;
    }

    public int getProteinNameColumn() {
        return proteinNameColumn;
    }

    public void setFileName(String fn) {
        this.filename = fn;
    }

    public String getParserName() {
        return ParserName;
    }

    public float getParserVersion() {
        return ParserVersion;
    }

    public void setProtsAndPeps(HashMap<String, ArrayList<QuantifiedAminoAcidSequence>> ProtsAndPeps) {
        this.ProtsAndPeps = ProtsAndPeps;
    }

    public HashMap getProtsAndPeps() {

        return ProtsAndPeps;
    }

    // This getHeaders functions returns all headers found as defined in the parser. A generic parser would return more or less headers.
    public ArrayList<String[]> getHeaders() {

        return headers;
    }

    public boolean isUseUniquePeptidesOnly() {
        return useUniquePeptidesOnly;
    }

    public void setUseUniquePeptidesOnly(boolean useUniquePeptidesOnly) {
        this.useUniquePeptidesOnly = useUniquePeptidesOnly;
    }

    public int headerIndex(String headerName) {
        String[] index = this.headers.get(1);
        int found = 0;
        for (int t = 0; t < index.length; t++) {
            if (index[t].equals(headerName)) {
                found = t;
            }
        }
        return found;
    }

    private void updateColumnInfo() {
        if (this.setColumns) {
            System.out.println("Finding columns...");
            this.peptideColumn = this.headerIndex("Sequence");
            this.modificationColumn = this.headerIndex("Modifications");
            this.peptideSignalA = this.headerIndex("Intensity 1");
            this.peptideSignalB = this.headerIndex("Intensity 2");
            this.proteinNameColumn = this.headerIndex("Accession number");
            System.out.println(this.peptideColumn + " " + this.modificationColumn + " " + this.peptideSignalA + " " + this.peptideSignalB);
        }
    }

    public void determineColumnsFromFirstRead() throws IOException {
        String line;
        String protein = "";
        MSQfile = new java.io.File(filename);
        boolean Peplines = false;
        RF = new RandomAccessFile(MSQfile, "r");
        search:
        while ((line = RF.readLine()) != null) {
            line = line.trim();
            String[] Line_array = line.split("\t");

            //      Pattern pattern = Pattern.compile(".*Accession number.*");
            Pattern pattern = Pattern.compile(".*Charge.*");
            Matcher matcher = pattern.matcher(line);
            if (matcher.matches()) {
                this.headers.add(0, line.split("\t"));
                this.headers.add(1, line.split("\t"));
                // System.out.println(hline);
                break search;
            }
        }
        // we have the header.. lets determine the actual header columns
        this.peptideColumn = this.headerIndex("Sequence");
        this.modificationColumn = this.headerIndex("Modifications");
        this.peptideSignalA = this.headerIndex("Intensity 1");
        this.peptideSignalB = this.headerIndex("Intensity 2");
        this.proteinNameColumn = this.headerIndex("Accession number");
        this.proteinDescriptionColumn = this.headerIndex("Description");
    }

    private void fillMods() {
        ArrayList<String> AL = new ArrayList<String>();

//        n-dim2, dim2K, 

        AL = new ArrayList<String>();
        AL.add("M");
        mods.put("oxM", AL);
        symbol.put("oxM", "(ox)");

        AL = new ArrayList<String>();
        AL.add("S");
        AL.add("T");
        mods.put("pST", AL);
        symbol.put("pST", "p");

        AL = new ArrayList<String>();
        AL.add("Y");
        mods.put("pY", AL);
        symbol.put("pY", "p");

        AL = new ArrayList<String>();
        AL.add("Nterm");
        mods.put("n-dim2", AL);
        symbol.put("n-dim2", "Dimeth2-");

        AL = new ArrayList<String>();
        AL.add("K");
        mods.put("dim2K", AL);
        symbol.put("dim2K", "Dimeth2K");


        AL = new ArrayList<String>();
        AL.add("M");
        mods.put("Met(ox)", AL);
        symbol.put("Met(ox)", "(ox)");

        AL = new ArrayList<String>();
        AL.add("S");
        AL.add("T");
        mods.put("p(ST)", AL);
        symbol.put("p(ST)", "p");

        AL = new ArrayList<String>();
        AL.add("N");
        AL.add("Q");
        mods.put("deami(NQ)", AL);
        symbol.put("deami(NQ)", "(dea)");

        AL = new ArrayList<String>();
        AL.add("R");
        mods.put("Arg10", AL);
        symbol.put("Arg10", "");

        AL = new ArrayList<String>();
        AL.add("L");
        mods.put("Lys8", AL);
        symbol.put("Lys8", "");

        AL = new ArrayList<String>();
        AL.add("Y");
        mods.put("p(Y)", AL);
        symbol.put("p(Y)", "p");

        AL = new ArrayList<String>();
        AL.add("Nterm");
        mods.put("Ac-N", AL);
        symbol.put("Ac-N", "Ac-");

        AL = new ArrayList<String>();
        AL.add("Nterm");
        mods.put("DiMeth_N", AL);
        symbol.put("DiMeth_N", "DiMeth-");

        AL = new ArrayList<String>();
        AL.add("Nterm");
        mods.put("DiMethD2_N", AL);
        symbol.put("DiMethD2_N", "DiMethD2-");

        AL = new ArrayList<String>();
        AL.add("S");
        AL.add("T");
        mods.put("Phospho_ST", AL);
        symbol.put("Phospho_ST", "p");

        AL = new ArrayList<String>();
        AL.add("K");
        mods.put("DiMethD2", AL);
        symbol.put("DiMethD2_K", "DiMethD2");

        AL = new ArrayList<String>();
        AL.add("K");
        mods.put("DiMeth", AL);
        symbol.put("DiMeth_K", "DiMeth");
    }

    private String modType(String mod, String AminoAcid, boolean nterm) {
        String modstring = "";

        if (nterm) {
            if (mods.get(mod).get(0).equals("Nterm")) {
                //   System.out.println(m.group(1)+" "+m.group(2));
                modstring = symbol.get(mod);
            }
        } else {
            String[] AA = mods.get(mod).toArray(new String[0]); // all AA associated to this modification
            for (String AminoA : AA) {
                // iterate through them
                if (AminoA.equals(AminoAcid)) {
                    // If one of the AA's matches with the AA of the query than it is found and can be used
                    modstring = symbol.get(mod);
                }
            }
        }
        return modstring;

    }

    private HashMap<String, Integer> removePTM(HashMap<String, Integer> NWMODS, String MODS) {

        Integer i = NWMODS.get(MODS);
        NWMODS.put(MODS, i - 1);
        //   System.out.print(" ["+MODS+"= "+i+" -> "+(i-1)+" ");
        if (i - 1 < 1) {
            //      System.out.print(" Removed "+MODS+"]  ");
            NWMODS.remove(MODS);
        }
        return NWMODS;
    }

    // this function parsers the
    private String parsePMTSequence(String seq, String[] modifications) {

        System.out.println(seq);

        //mods = array with eg.. 1P(ST) or 2P(ST)...

        /** THIS PART IS NOW DEPRICATED
         */
        Pattern pattern = Pattern.compile("\\*\\*\\*([A-Z])\\*\\*\\*");
        Matcher matcher = pattern.matcher(seq);

        //And check for ***,***
        Pattern pattern2 = Pattern.compile("\\*\\*\\*,\\*\\*\\*([A-Z])\\*\\*\\*\\,*\\*\\*");
        Matcher matcher2 = pattern2.matcher(seq);

        // check mod list for N-term !
        boolean hasNterm = false;
        boolean NtermFound = false;
        boolean isHeavyPeptide = false;



        HashMap<String, Integer> newMods = new HashMap<String, Integer>(); // hashed array of Modifications and amount
        for (String modis : modifications) {
            Pattern p = Pattern.compile("(\\d)(.*)");
            Matcher m = p.matcher(modis);
            if (m.matches()) {
                newMods.put(m.group(2), Integer.parseInt(m.group(1)));
            }
        }
        // Check if peptide contains a Nterm mod!!  IF not than you can disregards this peptide as valid
        Iterator it = newMods.keySet().iterator();
        while (it.hasNext()) {
            String modification = it.next().toString();
            if (this.mods.get(modification).get(0).equals("Nterm")) {
                hasNterm = true;
            }
        }
        if (newMods.containsKey("Lys8")) {
            isHeavyPeptide = true;
            newMods.remove("Lys8");
        }
        if (newMods.containsKey("Arg10")) {
            isHeavyPeptide = true;
            newMods.remove("Arg10");
        }

        if (newMods.containsKey("DiMeth13D3_N")) {
            isHeavyPeptide = true;
            newMods.remove("DiMeth13C3_N");
        }
        if (newMods.containsKey("DiMeth13C3_K")) {
            isHeavyPeptide = true;
            newMods.remove("DiMeth13C3_K");
        }

        if (newMods.containsKey("DiMeth_N")) {
            isHeavyPeptide = true;
            newMods.remove("DiMeth_N");
        }
        if (newMods.containsKey("DiMeth_K")) {
            isHeavyPeptide = true;
            newMods.remove("DiMeth_K");
        }
        if (newMods.containsKey("DiMethD2_N")) {
            isHeavyPeptide = true;
            newMods.remove("DiMethD2_N");
        }
        if (newMods.containsKey("DiMethD2_K")) {
            isHeavyPeptide = true;
            newMods.remove("DiMethD2_K");
        }

        while (matcher2.find()) {
            Nterm:
            if (hasNterm && matcher2.start() == 0) {
                Iterator it2 = newMods.keySet().iterator();
                while (it2.hasNext()) {
                    String MoD = (String) it2.next();
                    String ModAdd = modType(MoD, matcher2.group(1), true);
                    if (!ModAdd.equals("")) {
                        // mod found
                        seq = ModAdd + "***" + matcher2.group(1) + "***" + seq.substring(matcher2.end() + 1, seq.length());
                        newMods = this.removePTM(newMods, MoD);
                        hasNterm = false;
                        matcher2 = pattern2.matcher(seq);
                        matcher = pattern.matcher(seq);
                        break Nterm;
                    }
                }
            } else {
                Iterator it2 = newMods.keySet().iterator();
                loop:
                while (it2.hasNext()) {
                    String MoD = (String) it2.next();
                    String ModAdd = modType(MoD, matcher2.group(1), false);
                    if (!ModAdd.equals("")) {
                        // mod found
                        seq = seq.substring(0, matcher2.start()) + ModAdd + "***" + matcher2.group(1) + "***" + seq.substring(matcher2.end(), seq.length());
                        newMods = this.removePTM(newMods, MoD);
                        matcher2 = pattern2.matcher(seq);
                        matcher = pattern.matcher(seq);
                        break loop;
                    }
                }
            }
        }

        while (matcher.find()) {
            Nterm:
            if (hasNterm && matcher.start() == 0) {
                Iterator it2 = newMods.keySet().iterator();
                while (it2.hasNext()) {
                    String MoD = (String) it2.next();
                    String ModAdd = modType(MoD, matcher.group(1), true);
                    if (!ModAdd.equals("")) {
                        // mod found
                        seq = ModAdd + matcher.group(1) + seq.substring(matcher.end(), seq.length());
                        newMods = this.removePTM(newMods, MoD);
                        hasNterm = false;
                        matcher = pattern.matcher(seq);
                        break Nterm;
                    }
                }
            } else {
                Iterator it2 = newMods.keySet().iterator();
                loop:
                while (it2.hasNext()) {
                    String MoD = (String) it2.next();
                    String ModAdd = modType(MoD, matcher.group(1), false);
                    if (!ModAdd.equals("")) {
                        // mod found
                        seq = seq.substring(0, matcher.start()) + ModAdd + matcher.group(1) + seq.substring(matcher.end(), seq.length());
                        newMods = this.removePTM(newMods, MoD);
                        matcher = pattern.matcher(seq);
                        break loop;
                    }
                }
            }
        }


        /* It seems that the Latest version of MSQuant has a better formatting for PTM info. */

        if (isHeavyPeptide) {
            //    seq = seq + "--(heavy)";
        }
        if (newMods.size() > 0) {
            return "false";
        }

        return seq;
    }

    private void readFile() throws IOException {
        String line;
        String protein = "";
        this.fillMods();
        boolean Peplines = false;
        RF = new RandomAccessFile(MSQfile, "r");
        ArrayList<Object> RAFilePointers = new ArrayList<Object>();
        // RAFilePointers.add(RF.getFilePointer()); //start of doc. 1st line
        while ((line = RF.readLine()) != null) {
            RAFilePointers.add(RF.getFilePointer());
            line = line.trim();
            String[] Line_array = line.split("\t");

            Pattern pattern = Pattern.compile(".*Accession number.*");
            Matcher matcher = pattern.matcher(line);
            if (matcher.matches()) {
                // get Header
                Peplines = false;
                this.headers.add(0, line.split("\t"));
                this.headers.add(1, line.split("\t"));
                Peplines = true;
            } else {
                // get peptide AND protein data
                // NEW: These lines make it possible to use this parser for both MSQ type outputs
                if (Line_array.length < 20) {
                    Peplines = false;
                } else if (Line_array[0].equals("PROTEIN")) { // These two lines were implemented to accomodate MSQuant 1.5 jun 5th 2009 by BvB
                    Peplines = false;
                }
                if ((!(line.equalsIgnoreCase(""))) && Peplines) {
                    boolean pepOK = false;
                    float _sigA = Float.parseFloat(Line_array[this.peptideSignalA]);
                    float _sigB = Float.parseFloat(Line_array[this.peptideSignalB]);

                    // System.out.println("-> Reading signals)"+ _sigA +"  "+_sigB);

                    if (_sigA > 0f && _sigB > 0f) {
                        QAAS = new QuantifiedAminoAcidSequence();
                        // The next part, creates a unique name for "proteins". If people decide to use the peptide sequence as a protein name than modifications
                        // will play a role to in the uniqueness of the name.
                        String PTMinfo = "";
                        String finalPTMsequence = "";

                        if (this.PTMpositionColNumber > 0) {
                            // this.setUsePeptideReport(true); // This actually means that you will use the Sequence with mods as Identifier
                            // did we use PTMposition info?
                            PTMinfo = Line_array[this.PTMpositionColNumber];
                            String Mod = Line_array[this.modificationColumn];
                            //Q1 Are there any mods?
                            if (!(PTMinfo.equals(""))) {
                                // Q2, how many mods?
                                String[] Mods = Mod.split(" ");
                                // Q3, parse the PTMinfo
                                String[] PTM = PTMinfo.split(" ,");
                                //  ArrayList<String[]> PTMINFO = new ArrayList<String[]>();

                                String newseq = "false";

                                boolean found = false;
                                for (int t = 0; t < PTM.length - 1; t++) {
                                    // Check all possible checked PTM's use the first best solution.
                                    String[] PTMsplit = PTM[t].split(" ");
                                  //  newseq = parsePMTSequence(PTMsplit[1], Mods);
                                    newseq = PTMsplit[1]+" score: "+PTMsplit[0].replace("\"", "");
                                    if (!(newseq.equals("false")) && !found) {
                                        found = true;
                                        finalPTMsequence = newseq;
                                        QAAS.getSequenceWithMods().add(finalPTMsequence);
                                    } else if ((!newseq.equals("false")) && found) {
                                        QAAS.getSequenceWithMods().add(newseq);
                                    }
                                }
                                if (found) {
                                    //DEBUG: // System.out.println(finalPTMsequence);
                                } else {
                                    finalPTMsequence = Line_array[this.peptideColumn] + Mod;
                                }
                            } else {
                                finalPTMsequence = Line_array[this.peptideColumn];
                            }
                        }

                        if (!this.usePeptideReport) {
                            protein = Line_array[this.proteinNameColumn];
                        } else {
                            if (this.PTMpositionColNumber > 0) {
                                protein = finalPTMsequence;
                            } else {

                                /** This code part is added for simone to add modifications to the peptide.. to make them unique in the peptide list
                                 * 
                                 **/
                                String Mod = Line_array[this.modificationColumn];
                                String[] Mods = Mod.split(" ");
                                HashMap<String, Integer> newMods = new HashMap<String, Integer>(); // hashed array of Modifications and amount
                                for (String modis : Mods) {
                                    //System.out.println(modis);
                                    Pattern p = Pattern.compile("(\\d)(.*)");
                                    Matcher m = p.matcher(modis);
                                    if (m.matches()) {
                                        newMods.put(m.group(2), Integer.parseInt(m.group(1)));
                                    }
                                }
                                if (newMods.containsKey("Lys8")) {
                                    newMods.remove("Lys8");
                                }
                                if (newMods.containsKey("Arg10")) {
                                    newMods.remove("Arg10");
                                }

                                if (newMods.containsKey("DiMeth13CD3_N")) {
                                    newMods.remove("DiMeth13CD3_N");
                                }
                                if (newMods.containsKey("DiMeth13CD3_K")) {
                                    newMods.remove("DiMeth13CD3_K");
                                }

                                if (newMods.containsKey("DiMeth_N")) {
                                    newMods.remove("DiMeth_N");
                                }
                                if (newMods.containsKey("DiMeth_K")) {
                                    newMods.remove("DiMeth_K");
                                }
                                if (newMods.containsKey("DiMethD2_N")) {
                                    newMods.remove("DiMethD2_N");
                                }
                                if (newMods.containsKey("DiMethD2_K")) {
                                    newMods.remove("DiMethD2_K");
                                }
                                String modstringforoutput = "";
                                Iterator it = newMods.keySet().iterator();
                                while (it.hasNext()) {
                                    modstringforoutput += it.next().toString() + " , ";
                                }

                                protein = Line_array[this.proteinNameColumn] + "  " + modstringforoutput; //simone version
                                //  protein = Line_array[this.proteinNameColumn];
                            }
                        }

                        // does this protein already exists? if not create new hash entry
                        if (!(ProteinInfo.containsKey(protein))) {
                            ProteinInfo.put(protein, Line_array);
                        }

                        if (ProtsAndPeps.containsKey(protein)) {
                            // Check if peptide already exists for this protein.
                            if (this.useUniquePeptidesOnly) {
                                boolean unique = true;
                                uniquepeptide:
                                for (int t = 0; t < ProtsAndPeps.get(protein).size(); t++) {
                                    if (ProtsAndPeps.get(protein).get(t).getSequence().equals(Line_array[this.peptideColumn])) {
                                        // next make sure that it has the same modifications
                                        if (ProtsAndPeps.get(protein).get(t).getModification().equals(Line_array[this.modificationColumn])) {
                                            // ADD AREAS
                                            String[] pepinfo = ProtsAndPeps.get(protein).get(t).getPeptideinfo();
                                            float sigA = Float.parseFloat(Line_array[this.peptideSignalA]);
                                            float sigB = Float.parseFloat(Line_array[this.peptideSignalB]);
                                            sigA += Float.parseFloat(pepinfo[this.peptideSignalA]);
                                            sigB += Float.parseFloat(pepinfo[this.peptideSignalB]);
                                            pepinfo[this.peptideSignalA] = Float.toString(sigA);
                                            pepinfo[this.peptideSignalB] = Float.toString(sigB);
                                            ProtsAndPeps.get(protein).get(t).setPeptideinfo(pepinfo);
                                            float ratioBA = Float.parseFloat(pepinfo[this.peptideSignalB]) / Float.parseFloat(pepinfo[this.peptideSignalA]);
                                            ProtsAndPeps.get(protein).get(t).setRatio(ratioBA);
                                            unique = false;
                                            break uniquepeptide;
                                        }
                                    }
                                }
                                // if uniue !!
                                if (unique) {
                                    QAAS.setSequence(Line_array[this.peptideColumn]);
                                    QAAS.setPeptideinfo(Line_array);
                                    QAAS.setModification(Line_array[this.modificationColumn]);
                                    float ratioBA = Float.parseFloat(Line_array[this.peptideSignalB]) / Float.parseFloat(Line_array[this.peptideSignalA]);
                                    QAAS.setRatio(ratioBA);
                                    if (this.PTMpositionColNumber > 0) {
                                        QAAS.setsequenceWithMods(finalPTMsequence);

                                    }
                                    ProtsAndPeps.get(protein).add(QAAS);
                                }

                                // END UNIQUE*/
                            } else {
                                QAAS.setSequence(Line_array[this.peptideColumn]);
                                QAAS.setPeptideinfo(Line_array);
                                QAAS.setModification(Line_array[this.modificationColumn]);
                                float ratioBA = Float.parseFloat(Line_array[this.peptideSignalB]) / Float.parseFloat(Line_array[this.peptideSignalA]);
                                QAAS.setRatio(ratioBA);
                                if (this.PTMpositionColNumber > 0) {
                                    QAAS.setsequenceWithMods(finalPTMsequence);
                                }
                                ProtsAndPeps.get(protein).add(QAAS);
                            }
                        } else {
                            //new protein
                            QAAS.setSequence(Line_array[this.peptideColumn]);
                            QAAS.setPeptideinfo(Line_array);
                            QAAS.setModification(Line_array[this.modificationColumn]);
                            float ratioBA = Float.parseFloat(Line_array[this.peptideSignalB]) / Float.parseFloat(Line_array[this.peptideSignalA]);
                            QAAS.setRatio(ratioBA);
                            ArrayList<QuantifiedAminoAcidSequence> QAASlist = new ArrayList<QuantifiedAminoAcidSequence>();
                            QAASlist.add(QAAS);
                            if (this.PTMpositionColNumber > 0) {
                                QAAS.setsequenceWithMods(finalPTMsequence);
                            }
                            ProtsAndPeps.put(protein, QAASlist);
                        }
                    } //latest
                }
            }
        }
    }
}
